@@ -62,3 +62,8 @@ class MultiIndex(BaseIndex):
     def __repr__(self):
         return f"MultiIndex({self.indexes})"
 
+    def label_options(self, query):
+        label_options = []
+        for idx in self.indexes:
+            label_options.append(idx.label_options(query))
+        return label_options
\ No newline at end of file
