@@ -1,17 +1,26 @@
+from functools import singledispatch
 from itertools import product
+import numbers
 from typing import List, Union
 from warnings import warn
 from loguru import logger
 
 import pandas as pd
 from pydantic import BaseModel
+import datetime
 
 from ..types import Interval
 
 from ..indexes import Index, InterpolatingIndex, IntervalIndex, MultiIndex
-from ..utils import singledispatchmethod
+from ..utils import jsonable, singledispatchmethod
 from .base import BaseDataQuery, DatasourceInterface
 
+query_precendence = {
+    Index: 1,
+    IntervalIndex: 2,
+    InterpolatingIndex:3, 
+}
+
 try:
     import pymongo
     from pymongo.collection import Collection
@@ -115,7 +124,7 @@ try:
                 if len(docs) >= self.docs_per_label:
                     docs = self.index.reduce(docs, self.labels)
                     for doc in docs:
-                        yield doc
+                        yield from_mongo(doc)
                         collected += 1
                         if collected >= limit:
                             return
@@ -123,7 +132,7 @@ try:
             if len(docs) and collected<limit:
                 docs = self.index.reduce(docs, self.labels)
                 for doc in docs:
-                    yield doc
+                    yield from_mongo(doc)
                     collected += 1
                     if collected >= limit:
                         return
@@ -143,6 +152,9 @@ try:
                     
                 results[field] = [doc['first'] for doc in 
                                   self.collection.aggregate(pipeline, allowDiskUse=True)]
+
+            results = from_mongo(results)
+
             if len(fields) == 1:
                 return results[fields[0]]
             return results
@@ -159,9 +171,13 @@ try:
                 pipeline.append({"$limit": 1})
                 pipeline.append({"$project": { "_id": 0}})
                 try:
+
                     results[field] = next(self.collection.aggregate(pipeline, allowDiskUse=True))[field]
-                except StopIteration:
+                except (StopIteration, KeyError):
                     results[field] = None
+
+            results = from_mongo(results)
+
             if len(fields) == 1:
                 return results[fields[0]]
             return results
@@ -179,9 +195,11 @@ try:
                 pipeline.append({"$project": { "_id": 0}})
                 try:
                     results[field] = next(self.collection.aggregate(pipeline, allowDiskUse=True))[field]
-                except StopIteration:
+                except (StopIteration, KeyError):
                     results[field] = None
 
+            results = from_mongo(results)
+
             if len(fields) == 1:
                 return results[fields[0]]
             return results
@@ -245,8 +263,12 @@ try:
 
         def simple_multi_query(self, index, labels):
             pipeline = []
-
-            for idx, label in zip(index.indexes, labels.values()):
+            indexes = sorted(index.indexes,
+                             key=lambda x: query_precendence[type(x)])
+            for idx in indexes:
+                label = labels[idx.name]
+                if label is None:
+                    continue
                 others = [name for name in index.names if name != idx.name]
                 agg = self.compile_query(idx, label, others=others)
                 pipeline.extend(agg.pipeline)
@@ -277,6 +299,8 @@ try:
                         f'{index} with label: {label}')
             name = index.name if isinstance(index, Index) else index
 
+            label = to_mongo(label)
+
             if isinstance(label, slice):
                 # support basic slicing, this will only work
                 # for values that are comparable with the
@@ -320,6 +344,9 @@ try:
             """
             logger.debug('Building mongo interpolating-query for index: '
                         f'{index} with label: {label}')
+            
+            label = to_mongo(label)
+
             if label is None:
                 labels = {index.name: label}
                 return MongoAggregation(index, labels, self.source, [])
@@ -358,6 +385,8 @@ try:
                 intervals = label
             else:
                 intervals = [label]
+            
+            intervals = to_mongo(intervals)
 
             queries = []
             for interval in intervals:
@@ -399,11 +428,11 @@ try:
             avoid replacing existing documents with a copy.
             """
             from rframe.schema import InsertionError
-
+            index = to_mongo(doc.index_labels)
             try:
                 doc = self.source.find_one_and_update(
-                    doc.index_labels,
-                    {"$set": doc.dict()},
+                    index,
+                    {"$set": to_mongo(doc.dict())},
                     projection={"_id": False},
                     upsert=True,
                     return_document=pymongo.ReturnDocument.AFTER,
@@ -418,7 +447,8 @@ try:
             self.source.ensure_index([(name, order) for name in names])
 
         def delete(self, doc):
-            return self.source.delete_one(doc.index_labels)
+            index = to_mongo(doc.index_labels)
+            return self.source.delete_one(index)
 
         def initdb(self, schema):
             index_names = list(schema.get_index_fields())
@@ -598,3 +628,73 @@ def merge_pipelines(pipelines):
         
     ]
     return pipeline
+
+
+@singledispatch
+def to_mongo(obj):
+    return obj
+
+@to_mongo.register(dict)
+def to_mongo_dict(obj: dict):
+    return {k: to_mongo(v) for k, v in obj.items()}
+
+@to_mongo.register(list)
+def to_mongo_list(obj):
+    return [to_mongo(v) for v in obj]
+
+@to_mongo.register(tuple)
+def to_mongo_tuple(obj):
+    return tuple(to_mongo(v) for v in obj)
+
+@to_mongo.register(BaseModel)
+def to_mongo_interval(obj):
+    return to_mongo(obj.dict())
+
+@to_mongo.register(pd.DataFrame)
+def to_mongo_df(df):
+    return to_mongo(df.to_dict(orient="records"))
+
+@to_mongo.register(datetime.datetime)
+def to_mongo_datetime(obj):
+    # mongodb datetime has millisecond resolution
+    return obj.replace(microsecond=int(obj.microsecond/1000)*1000)
+
+@to_mongo.register(datetime.timedelta)
+def to_mongo_timedelta(obj):
+    # mongodb datetime has millisecond resolution
+    seconds = int(obj.total_seconds()*1e3)/1e3
+    return datetime.timedelta(seconds=seconds)
+
+@to_mongo.register(pd.Timestamp)
+def to_mongo_timestamp(obj):
+    return to_mongo(obj.to_pydatetime())
+
+@to_mongo.register(pd.Timedelta)
+def to_mongo_timedelta(obj):
+    return to_mongo(obj.to_pytimedelta())
+
+@to_mongo.register(numbers.Integral)
+def to_mongo_int(obj):
+    return int(obj)
+
+@to_mongo.register(numbers.Real)
+def to_mongo_float(obj):
+    return float(obj)
+
+@singledispatch
+def from_mongo(obj):
+    return obj
+
+@from_mongo.register(list)
+def from_mongo_list(obj):
+    return [from_mongo(v) for v in obj]
+
+@from_mongo.register(tuple)
+def from_mongo_tuple(obj):
+    return tuple(from_mongo(v) for v in obj)
+
+@from_mongo.register(dict)
+def from_mongo_dict(obj):
+    if len(obj)==2 and 'left' in obj and 'right' in obj:
+        return Interval[obj['left'],obj['right']]
+    return {k: from_mongo(v) for k, v in obj.items()}
