@@ -1,22 +1,25 @@
+from functools import singledispatch
 import fsspec
 import json
 import numpy as np
 import pandas as pd
 from itertools import product
 from typing import Any, Dict, List, Optional, Union
-from warnings import warn
 from loguru import logger
 
 
 from pydantic import BaseModel
 
-from ..types import Interval
+from .json import from_json, to_json
 
 from ..indexes import Index, InterpolatingIndex, IntervalIndex, MultiIndex
 from ..utils import jsonable, singledispatchmethod, hashable_doc, unhashable_doc
 
 from .base import BaseDataQuery, DatasourceInterface
 
+to_tinydb = to_json
+from_tinydb = from_json
+
 try:
     from tinydb import TinyDB, Query, where
     from tinydb.table import Table
@@ -71,7 +74,7 @@ try:
             if not isinstance(docs, list):
                 docs = [docs]
                 
-            return docs
+            return from_tinydb(docs)
 
     class TinyDBQuery(BaseDataQuery):
         def __init__(self, index, labels, table, selections) -> None:
@@ -323,8 +326,6 @@ except ImportError:
         def __init__(self) -> None:
             raise TypeError("Cannot use TinyDB interface. tindb not installed.")
 
-    warn("tinydb not found, cannot register TinDB interface.")
-
 
 def overlap_query(target, left, right):
     query = target.noop()
@@ -350,4 +351,4 @@ def extract_interval_edges(interval):
     else:
         left = right = interval
 
-    return left, right
\ No newline at end of file
+    return left, right
