@@ -1,12 +1,17 @@
 """Main module."""
 
+import pandas as pd
+
 from datetime import datetime
 from typing import Any, List, Tuple, Type, Union
+from pandas.core.indexes.frozen import FrozenList
 
-import pandas as pd
 from pydantic.typing import NoneType
 
+from .indexes import MultiIndex
+
 from .interfaces import get_interface
+from .interfaces.pandas import to_pandas
 from .schema import BaseSchema, InsertionError, UpdateError
 from .utils import camel_to_snake, singledispatchmethod
 
@@ -20,6 +25,7 @@ class RemoteFrame:
 
     schema: Type[BaseSchema]
     db: Any
+    df: pd.DataFrame
 
     _index = None
 
@@ -27,6 +33,7 @@ class RemoteFrame:
         self.schema = schema
         self._datasource = datasource
         self._labels = labels
+        self._index = None
 
     @classmethod
     def from_mongodb(cls, schema, url, db, collection, **kwargs):
@@ -50,12 +57,17 @@ class RemoteFrame:
     def name(self):
         return camel_to_snake(self.schema.__name__)
 
+    @property
+    def index_names(self):
+        return FrozenList(self.schema.get_index_fields())
+
     @property
     def columns(self):
         return pd.Index(list(self.schema.get_column_fields()))
 
     @property
     def index(self):
+        # FIXME: make a proxy object that only loads labels if accessed.
         if self._index is None:
             self._recreate_index()
         return self._index
@@ -119,7 +131,7 @@ class RemoteFrame:
         returns a pandas dataframe if all indexes are specified.
         Otherwise returns a RemoteFrame
         """
-        index_fields = self.index.names
+        index_fields = self.index_names
         labels = {name: lbl for name, lbl in zip(index_fields, args)}
         labels.update(kwargs)
 
@@ -132,7 +144,7 @@ class RemoteFrame:
                 merged[k] = v
 
         rf = RemoteFrame(self.schema, self.datasource, **merged)
-        if all([label in merged for label in self.index.names]):
+        if all([label in merged for label in self.index_names]):
             rf = rf.df
         if len(extra):
             rf = RemoteFrame(self.schema, rf, **extra)
@@ -141,7 +153,7 @@ class RemoteFrame:
 
     def set(self, *args: IndexLabel, **kwargs: IndexLabel) -> BaseSchema:
         """Insert data by index"""
-        labels = {name: lbl for name, lbl in zip(self.index.names, args)}
+        labels = {name: lbl for name, lbl in zip(self.index_names, args)}
         kwargs.update(labels)
         doc = self.schema(**kwargs)
         res = doc.save(self.datasource)
@@ -174,18 +186,15 @@ class RemoteFrame:
         return succeeded, failed, errors
 
     def _recreate_index(self):
-        fields = list(self.schema.get_index_fields())
-        unique = self.unique(fields)
-        if isinstance(unique, dict):
-            values = []
-            for k,vs in unique.items():
-                index = self.schema.index_for(k)
-                values.append([index.to_pandas(vi) for vi in vs])
-            self._index =  pd.MultiIndex.from_product(values, names=fields)
+        index = self.schema.get_index()
+        query = self.schema.compile_query(self.datasource,
+                                          **self._labels)
+        label_options = to_pandas(index.label_options(query))
+
+        if isinstance(index, MultiIndex):
+            self._index =  pd.MultiIndex.from_product(label_options, names=index.names)
         else:
-            index = self.schema.index_for(fields[0])
-            unique = [index.to_pandas(vi) for vi in unique]
-            self._index = pd.Index(unique, name=fields[0])
+            self._index =  pd.Index(label_options, name=index.name)
         
     def __getitem__(self, index: Tuple[IndexLabel, ...]) -> "RemoteSeries":
         if isinstance(index, str) and index in self.columns:
@@ -195,7 +204,7 @@ class RemoteFrame:
         raise KeyError(f"{index} is not a dataframe column.")
 
     def __call__(self, column: str, **index: IndexLabel) -> pd.DataFrame:
-        index = tuple(index.get(k, None) for k in self.index.names)
+        index = tuple(index.get(k, None) for k in self.index_names)
         return self.at[index, column]
 
     def __dir__(self) -> List[str]:
@@ -204,7 +213,7 @@ class RemoteFrame:
     def __getattr__(self, name: str) -> "RemoteSeries":
         if name != "columns" and name in self.columns:
             return self[name]
-        raise AttributeError(name)
+        return super().__getattribute__(name)
 
     def __len__(self):
         return self.size
@@ -212,7 +221,7 @@ class RemoteFrame:
     def __repr__(self) -> str: # pragma: no cover
         return ( 
             f"{self.__class__.__name__}("
-            f"index={self.index.names},"
+            f"index={self.index_names},"
             f"columns={self.columns},"
             f"datasource={self._datasource},"
             f"selection={self._labels})"
