@@ -107,7 +107,7 @@ class BaseTestSchema(BaseSchema):
             for field, value in doc.column_values.items():
                 assert are_equal(value, rf[field].at[doc.index_labels_tuple])
         
-        df = pd.concat([doc.to_pandas() for doc in docs])
+        df = pd.concat([doc.dframe() for doc in docs])
 
         tester.assertEqual(len(rf), len(df))
 
@@ -222,7 +222,7 @@ class IntervalTestSchema(BaseTestSchema):
     def list_strategy(cls, **overrides):
         defaults = dict(
             unique_by=lambda x: x.index_field.left,
-            min_size=2, max_size=20,
+            min_size=2, max_size=10,
         )
         kwargs = dict(defaults, **overrides)
         return st.lists(cls.item_strategy(), **kwargs)
